<div class="panel panel-default">
    <div class="panel-heading">Storage Compiling Status</div>
    <div class="panel-body">
        <table class="table">
            <thead>
                <tr>
                    <th></th>
                    <th>Is Running</th>
                    <th>Queue Size</th>
                    <th>Total Processed</th>
                    <th>Last Error</th>
                </tr>
            </thead>

            <tbody>
                <tr>
                    <th>Document Storage Compiler</th>
                    <td>{{ item.documentCompilerQueueStatus }}</td>
                    <td>{{ item.documentCompilerQueueSize }}</td>
                    <td>{{ item.documentCompilerQueueProcessedTotal }}</td>
                    <td>{{ item.documentCompilerQueueLastError }}</td>
                </tr>
            </tbody>
        </table>
    </div>
</div>
