@import "~@synerty/peek-plugin-base-js/src/styles/_mixins";
@import "~@synerty/peek-plugin-base-js/src/styles/_variables";

$margin: 8px;

.docdb-popup {
    position: fixed;
    z-index: 999;
    box-shadow: 0 2px 4px rgba(0, 0, 0, 0.3);
}

.buttons {
    margin-top: $margin;
    display: flex;
}

.dynamic-buttons {
    flex-grow: 1;
}

.dynamic-buttons > button {
    margin-right: $margin;
}

.details-buttons > button {
    margin-left: $margin;
}

.child-menu-items {
    height: 400px;
    overflow-y: scroll;
}
